@@ -2,19 +2,18 @@
  * Copyright (C) 2009,2010 Red Hat, Inc.
  * Copyright (C) 2013 Google, Inc.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This library is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published
+ * by the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library.  If not, see <https://www.gnu.org/licenses/>.
  *
  * Red Hat Author(s): Behdad Esfahbod
  * Google Author(s): Behdad Esfahbod
@@ -127,6 +126,10 @@ typedef guint8 _vte_overwrite_counter_t;
 
 /******************************************************************************************/
 
+#ifndef HAVE_EXPLICIT_BZERO
+#define explicit_bzero(s, n) memset((s), 0, (n))
+#endif
+
 #ifndef HAVE_PREAD
 #define pread _pread
 static inline gsize
@@ -557,9 +560,9 @@ _vte_snake_advance_tail (VteSnake *snake, gsize offset)
                                 break;
                         }
                 }
+                snake->tail = snake->segment[0].st_tail;
         }
-        g_assert_cmpuint (snake->segment[0].st_tail, ==, offset);
-        snake->tail = offset;
+        g_assert_cmpuint (snake->tail, ==, offset);
 }
 
 static gsize
@@ -830,12 +833,10 @@ _vte_boa_init (VteBoa *boa)
         datum_key.data = key;
         datum_key.size = VTE_CIPHER_KEY_SIZE;
         gnutls_cipher_init(&boa->cipher_hd, VTE_CIPHER_ALGORITHM, &datum_key, NULL);
-        /* FIXME: 738601#c52 the compiler might optimize this away, how to make sure it's erased?
-         * It's on the stack so maybe we can rest assured it'll be overwritten pretty soon. */
-        memset(key, 0, VTE_CIPHER_KEY_SIZE);
+        explicit_bzero(key, VTE_CIPHER_KEY_SIZE);
 
         /* Empty IV. */
-        memset(&boa->iv, 0, sizeof(boa->iv));
+        explicit_bzero(&boa->iv, sizeof(boa->iv));
 #endif
 
         boa->compressBound = _vte_boa_compressBound(VTE_BOA_BLOCKSIZE);
@@ -847,7 +848,7 @@ _vte_boa_finalize (GObject *object)
 #if !defined VTESTREAM_MAIN && defined WITH_GNUTLS
         VteBoa *boa = (VteBoa *) object;
 
-        memset(&boa->iv, 0, sizeof(boa->iv));
+        explicit_bzero(&boa->iv, sizeof(boa->iv));
 
         gnutls_cipher_deinit (boa->cipher_hd);
         gnutls_global_deinit ();
@@ -929,8 +930,6 @@ _vte_boa_read (VteBoa *boa, gsize offset, char *data)
 static void
 _vte_boa_write (VteBoa *boa, gsize offset, const char *data)
 {
-        _vte_block_datalength_t compressed_len = boa->compressBound;
-
         /* The overwrite counter is 1-based.  This is to make sure that the IV is never 0: 738601#c88,
            to make sure that an empty block (e.g. after a previous write failure) is always invalid,
            and to make unit testing easier */
@@ -964,6 +963,8 @@ _vte_boa_write (VteBoa *boa, gsize offset, const char *data)
                 overwrite_counter++;
         }
 
+        _vte_block_datalength_t compressed_len;
+
         /* Compress, or copy if uncompressable */
         compressed_len = _vte_boa_compress (buf + VTE_BLOCK_DATALENGTH_SIZE + VTE_OVERWRITE_COUNTER_SIZE, boa->compressBound,
                                             data, VTE_BOA_BLOCKSIZE);
@@ -1246,6 +1247,10 @@ G_END_DECLS
 
 #ifdef VTESTREAM_MAIN
 
+#ifdef __clang__
+#pragma clang diagnostic ignored "-Wstring-plus-int"
+#endif
+
 /* Some helpers. Macros rather than functions to report useful line numbers on failure. */
 
 /* Check for the file's exact contents */
